# Copyright (c) 2002-2005 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: AutoTOC.py,v 1.12 2005/08/05 16:00:26 kitblake Exp $

# Zope
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Persistence import Persistent
from OFS.SimpleItem import SimpleItem
from types import StringType

# products
from Products.ParsedXML.ParsedXML import ParsedXML

# Silva
from Products.Silva.Content import Content
from Products.Silva import SilvaPermissions
from Products.Silva import mangle
from Products.Silva.i18n import translate as _
from Products.Silva.interfaces import IContent, IContainerPolicy
from Products.Silva.interfaces import IContainer,IAsset
from Products.Silva.helpers import add_and_edit
from Products.Silva.ImporterRegistry import get_xml_id, get_xml_title
from Products.Silva.Metadata import export_metadata

icon = "www/autotoc.png"
addable_priority = 2

class CustomAutoTOC(Content, SimpleItem):
    __doc__ = _("""This is a special document that automagically displays a
       table of contents. Usually it&#8217;s used as an &#8216;index&#8217;
       document. In that case the parent folder shows a table of contents
       when accessed (e.g. http://www.x.yz/silva/myFolder/).  Custom AutoTOC
       can display any set of Silva Content Types (including assets).""")

    security = ClassSecurityInfo()

    meta_type = "Silva Custom AutoTOC"

    __implements__ = IContent

    def __init__(self, id):
        CustomAutoTOC.inheritedAttribute('__init__')(self, id,
            '[Title is stored in metadata. This is a bug.]')
        self._local_types = ['Silva Document','Silva Publication','Silva Folder']
        self._depth = -1 #unlimited depth

    # ACCESSORS
    security.declareProtected(SilvaPermissions.View, 'is_cacheable')
    def is_cacheable(self):
        return 0

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'is_deletable')
    def is_deletable(self):
        """always deletable"""
        return 1

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'can_set_title')
    def can_set_title(self):        
        """always settable"""
        # XXX: we badly need Publishable type objects to behave right.
        return 1

    security.declareProtected(SilvaPermissions.ReadSilvaContent, 'get_silva_types')
    def get_silva_types(self):
        st = self.get_silva_addables_allowed_in_publication()
        return st

    security.declareProtected(SilvaPermissions.View, 'get_local_types')
    def get_local_types(self):
        return self._local_types

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'set_local_types')
    def set_local_types(self, types):
        self._local_types = types
                    
    security.declareProtected(SilvaPermissions.View, 'get_depth')
    def get_depth(self):
        return self._depth

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'set_depth')
    def set_depth(self, depth):
        self._depth = depth

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_tree_iterator')
    def get_tree_iterator(self, container=None, indent=0):
        """yield for every element in this toc
        The 'depth' argument limits the number of levels, defaults to unlimited
        """
        if not container:
            container = self.get_container()
        items = container.objectItems(self._local_types)
        items.sort()
        for (item,obj) in items:
            if indent and item == 'index': #include the containers index
                # default document should not be inserted
                continue
            yield (indent, obj)
            if IContainer.isImplementedBy(obj) and \
                   obj.is_transparent() and \
                   (self._depth == -1 or indent < self._depth):
                for (dep,o) in self.get_tree_iterator(obj, indent + 1):
                    yield (dep,o)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_public_tree')
    def get_public_tree_iterator(self, container=None, indent=0, include_non_transparent_containers=0):
        if not container:
            container = self.get_container()
        items = container.objectItems(self._local_types)
        items.sort()
        for (name,item) in items:
            if not (item.is_published() or IAsset.isImplementedBy(item)) or \
                   (indent and name=='index'):
                continue
            if self.service_metadata.getMetadataValue(
                item.get_viewable(),
                'silva-extra',
                'hide_from_tocs') == 'hide':
                    continue
            yield (indent, item)
            if (IContainer.isImplementedBy(item) and \
                (item.is_transparent() or \
                 include_non_transparent_containers))  and \
                (self._depth == -1 or indent < self._depth):
                for (dep,o) in self.get_public_tree_iterator(item, indent+1,
                                                   include_non_transparent_containers):
                    yield (dep,o)

    security.declareProtected(SilvaPermissions.View, 'render_tree')
    def render_tree(self, public=0, append_to_url=None):
        if isinstance(append_to_url,StringType) and append_to_url[0] != '/':
            append_to_url = '/' + append_to_url
        else:
            append_to_url = ''

        func = public and self.get_public_tree_iterator or self.get_tree_iterator
        html = []
        a_templ = '<a class="toc-link" href="%s">%s</a>'

        prev_depth = [-1]
        for (depth,item) in func():
            pd = prev_depth[-1]
            if pd < depth: #down one level
                html.append('<ul class="custom-toc">')
                prev_depth.append(depth)
            elif pd > depth: #up one level
                for i in range(pd-depth):
                    prev_depth.pop()
                    html.append('</ul></li>')
            elif pd == depth: #same level
                html.append('</li>')
            html.append('<li>')
            title = (public and item.get_title() or item.get_title_editable()) or item.id
            html.append(a_templ%(item.absolute_url()+append_to_url,title ))
        else:
            html.append('</ul>')
        return '\n'.join(html)

    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                              'to_xml')
    def to_xml(self, context):
        """Render object to XML."""
        f = context.f
        f.write('<silva_custom_auto_toc id="%s" types="%s" depth="%i">' %( self.id, ','.join(self.get_local_types() ), self.get_depth() ) )
        f.write('<title>%s</title>' % self.get_title())
        export_metadata(self, context)
        f.write('</silva_custom_auto_toc>')
InitializeClass(CustomAutoTOC)

manage_addCustomAutoTOCForm = PageTemplateFile("www/customAutoTOCAdd", globals(),
    __name__='manage_addCustomAutoTOCForm')

def manage_addCustomAutoTOC(self, id, title, REQUEST=None):
    """Add a Custom AutoTOC."""
    if not mangle.Id(self, id).isValid():
        return
    object = CustomAutoTOC(id)
    self._setObject(id, object)
    object = getattr(self, id)
    object.set_title(title)
    add_and_edit(self, id, REQUEST)
    return ''

class CustomAutoTOCPolicy(Persistent):

    __implements__ = IContainerPolicy

    def createDefaultDocument(self, container, title):
        container.manage_addProduct['SilvaCustomAutoTOC'].manage_addCustomAutoTOC(
            'index', title)
        container.index.sec_update_last_author_info()

def xml_import_handler(object, node):
    id = get_xml_id(node)
    title = get_xml_title(node)
    id = str(mangle.Id(object, id).unique())
    object.manage_addProduct['SilvaCustomAutoTOC'].manage_addCustomAutoTOC(id, title)
    newdoc = getattr(object, id)
    newdoc.sec_update_last_author_info()
    depth = int(node.getAttribute('depth') or -1)
    newdoc.set_depth(depth)
    types = node.getAttribute('types').split(',')
    newdoc.set_local_types(types)
    return newdoc
